#include "stdafx.h"
#include "RegNotifyUtil.h"

/* Defines */
#define	AYGSHELL_DLL_NAME						_T("aygshell.dll")
#define REGISTRYNOTIFYCALLBACK_FUNC_NAME		_T("RegistryNotifyCallback")
#define REGISTRYCLOSENOTIFICATION_FUNC_NAME		_T("RegistryCloseNotification")

/* Typedefs */
typedef HRESULT (WINAPI * PFNREGISTRY_NOTIFY_CALLBACK)(HKEY, LPCTSTR, LPCTSTR, REGISTRYNOTIFYCALLBACK, DWORD, void*, HANDLE*);
typedef HRESULT (WINAPI * PFNREGISTRY_CLOSE_NOTIFICATION)(HANDLE);

typedef struct {
	PFNREGISTRY_NOTIFY_CALLBACK		pfnRegistryNotifyCallback;
	PFNREGISTRY_CLOSE_NOTIFICATION	pfnRegistryCloseNotification;
} REGNOTIFYFUNCENTRY;

/* Global Variables */
static	HMODULE				gs_hModule = NULL;
static	REGNOTIFYFUNCENTRY	gs_RNEntry;

static
BOOL
isInitialized()
{
	return (gs_hModule != NULL);
}

static
REGNOTIFYFUNCENTRY*
getEntry()
{
	return &gs_RNEntry;
}

BOOL
RNUtil_Initialize()
{
	BOOL		bRet;
	HMODULE		hModule = NULL;
	PFNREGISTRY_NOTIFY_CALLBACK		pfnRegistryNotifyCallback		= NULL;
	PFNREGISTRY_CLOSE_NOTIFICATION	pfnRegistryCloseNotification	= NULL;


	bRet = FALSE;	// default return value

	hModule = LoadLibrary(AYGSHELL_DLL_NAME);
	if (hModule != NULL) {
		pfnRegistryNotifyCallback		= (PFNREGISTRY_NOTIFY_CALLBACK)		GetProcAddress(hModule, REGISTRYNOTIFYCALLBACK_FUNC_NAME);
		pfnRegistryCloseNotification	= (PFNREGISTRY_CLOSE_NOTIFICATION)	GetProcAddress(hModule, REGISTRYCLOSENOTIFICATION_FUNC_NAME);

		if (pfnRegistryNotifyCallback != NULL &&
			pfnRegistryCloseNotification != NULL) {

			gs_hModule = hModule;

			gs_RNEntry.pfnRegistryNotifyCallback		= pfnRegistryNotifyCallback;
			gs_RNEntry.pfnRegistryCloseNotification	= pfnRegistryCloseNotification;

			bRet = TRUE;
		}
	}

	return bRet;
}


BOOL
RNUtil_Terminate()
{
	BOOL	bRet;

	bRet = FALSE;

	if (gs_hModule != NULL) {
		FreeLibrary(gs_hModule);
		gs_hModule = NULL;

		bRet = TRUE;
	}

	return bRet;
}


HRESULT
RNUtil_RegistryNotifyCallback(
	HKEY					hKey,
	LPCTSTR					pszSubKey,
	LPCTSTR					pszValueName,
	REGISTRYNOTIFYCALLBACK	pfnRegistryNotifyCallback,
	DWORD					dwUserData,
	void*					pCondition,
	HANDLE*					phNotify
)
{
	HRESULT	hr = E_FAIL;

	if (isInitialized()) {
		hr = (*(getEntry()->pfnRegistryNotifyCallback))(	hKey,
															pszSubKey,
															pszValueName,
															pfnRegistryNotifyCallback,
															dwUserData,
															pCondition,
															phNotify	);
	}

	return hr;
}


HRESULT
RNUtil_RegistryCloseNotification(
	HANDLE	hNotify
)
{
	HRESULT	hr = E_FAIL;

	if (isInitialized()) {
		hr = (*(getEntry()->pfnRegistryCloseNotification))(	hNotify );
	}

	return hr;
}



